<?php
/**
 * Plugin Name: WooCommerce TransFirst/TransactionCentral Gateway
 * Plugin URI: https://www.ecreations.net/product/woocommerce-transaction-central/
 * Description: Extends the WordPress WooCommerce e-commerce suite to provide a TransFirst (Transaction Central) payment gateway.
 * Version: 2015.03.05
 * Author: eCreations <admin@ecreations.net>
 * Author URI: http://www.ecreations.net/woocommerce-transaction-central/
 * License: Proprietary/Closed Source (see license.txt)
 *
 * In order to integrate Transaction Central with WooCommerce, you will need a merchant ID and reg key.
 * _These are not your login credentials._ To obtain them, you will need to:
 *
 * 1. Call TransFirst and verify that you are the account holder.
 * 2. They will ask what software you are going to integrate. Tell them WooCommerce.
 * 3. They will provide you with instructions to use the standard gateway Mijireh, which is a third-
 *    party service that acts as a bridge between WooCommerce and Transaction Central; you will need to
 *    create an account with Mijireh and follow their instructions.
 * 4. The TransFirst representative will provide you with a merchant ID and reg key (which they may
 *    refer to as a password) and you need to input these into Mijireh so that the technician can test
 *    that the credentials work.
 * 5. After this phone call, you can remove/deactivate your Mijireh account.
 *
 * Once you have obtained your merchant ID and reg key:
 *
 * 1. Extract this plugin and upload woocommerce-transfirst-gateway.php and transfirst.png to:
 *    <WordPress installation root>/wp-content/plugins/woocommerce-transfirst-gateway/
 * 2. Log into the WordPress Dashboard and go to Plugins > Installed Plugins.
 * 3. Click Activate under WooCommerce TransFirst Gateway.
 * 4. Go to WooCommerce > Settings > Payment Gateways > TransFirst.
 * 5. Check Enable. Enter your merchant ID and reg key and configure as you like. Save.
 *
 * On checkout, the user should be able to select TransFirst as a payment option and then provide credit
 * card information.
 *
 * NOTE: TransFirst does not provide its own interface; therefore, credit card and billing information
 * is passed via your website. Your website must have an SSL certificate installed, your checkout must
 * pass through HTTPS, and your server security must be PCI-compliant. The developers are not
 * responsible for misuse.
 */

if (!defined('ABSPATH')) {
	exit;
}

if (!function_exists('wc_transfirst_gateway_plugins_loaded') && !class_exists('WC_Gateway_TransFirst') && !function_exists('wc_transfirst_gateway_woocommerce_payment_gateways')) {
	/**
	 * Loads the gateway after plugins are loaded.
	 *
	 * @access public
	 * @return void
	 */
	function wc_transfirst_gateway_plugins_loaded () {
		if (!class_exists('WC_Payment_Gateway')) {
			if (!function_exists('wc_transfirst_gateway_admin_notices')) {
				function wc_transfirst_gateway_admin_notices () {
					?>
					<div class="error fade"><?php _e('The WooCommerce TransFirst Gateway plugin could not be loaded because WooCommerce is not installed and activated.', 'woocommerce_transfirst'); ?></div>
					<?php
				}
				add_action('admin_notices', 'wc_transfirst_gateway_admin_notices');
			}
			return;
		}

		/**
		 * WooCommerce TransFirst Gateway
		 *
		 * Extends the WordPress WooCommerce e-commerce suite to provide a TransFirst (Transaction
		 * Central) payment gateway.
		 *
		 * @class WC_Gateway_TransFirst
		 * @extends WC_Payment_Gateway
		 * @version 2014.02.09
		 * @package WooCommerce/Classes/Payment
		 * @author eCreations <http://www.ecreations.net/>
		 */
		class WC_Gateway_TransFirst extends WC_Payment_Gateway {
			/**
			 * Initializes an instance of the gateway class.
			 *
			 * @access public
			 * @return instance
			 */
			public function __construct () {
				global $woocommerce;

				$this->id = 'transfirst';
				$this->icon = apply_filters('woocommerce_transfirst_icon', plugin_dir_url(__FILE__).'transfirst.png');
				$this->has_fields = true;
				$this->method_title = __('TransFirst', 'woocommerce_transfirst');

				$this->init_form_fields();
				$this->init_settings();

				$this->title = $this->get_option('title');
				$this->description = $this->get_option('description');
				$this->merchant_id = $this->get_option('merchant_id');
				$this->reg_key = $this->get_option('reg_key');
				$this->invoice_prefix = $this->get_option('invoice_prefix', 'WC-');

				add_action('woocommerce_receipt_transfirst', array ($this, 'receipt_page'));
				add_action('woocommerce_update_options_payment_gateways_transfirst', array ($this, 'process_admin_options'));

				if (!$this->is_valid_for_use()) {
					$this->enabled = false;
				}
			}

			/**
			 * Checks if TransFirst is compatible with the currency used.
			 *
			 * @access public
			 * @return bool Whether or not it is compatible
			 */
			public function is_valid_for_use () {
				return in_array(get_woocommerce_currency(), apply_filters('woocommerce_transfirst_supported_currencies', array ('USD')));
			}

			/**
			 * Outputs options for dashboard gateway configuration.
			 *
			 * @access public
			 * @return void
			 */
			public function admin_options () {
				?>
				<h3><?php _e('TransFirst', 'woocommerce_transfirst'); ?></h3>
				<p><?php _e('The user enters credit card and billing information into a form. The data is passed to the TransFirst API for payment processing.', 'woocommerce_transfirst'); ?></p>
				<?php
				if ($this->is_valid_for_use()) {
				?>
				<table class="form-table">
					<?php $this->generate_settings_html(); ?>
				</table>
				<?php
				} else {
				?>
				<div class="inline error">
					<p><strong><?php _e('Gateway Disabled', 'woocommerce'); ?></strong>: <?php _e('TransFirst does not support your store currency.', 'woocommerce_transfirst'); ?></p>
				</div>
				<?php
				}
			}

			/**
			 * Initializes values and form fields for gateway settings.
			 *
			 * @access public
			 * @return void
			 */
			public function init_form_fields () {
				$this->form_fields = array (
					'enabled' => array (
						'title' => __('Enable/Disable', 'woocommerce'),
						'type' => 'checkbox',
						'label' => __('Enable TransFirst Gateway', 'woocommerce_transfirst'),
						'default' => 'yes'
					),
					'title' => array (
						'title' => __('Title', 'woocommerce'),
						'type' => 'text',
						'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
						'default' => __('TransFirst', 'woocommerce_transfirst'),
						'desc_tip' => true
					),
					'description' => array (
						'title' => __('Description', 'woocommerce'),
						'type' => 'textarea',
						'description' => __('This controls the description which the user sees during during checkout.', 'woocommerce'),
						'default' => __('Enter your credit card information below.', 'woocommerce_transfirst')
					),
					'merchant_id' => array (
						'title' => __('Merchant ID', 'woocommerce_transfirst'),
						'type' => 'text',
						'description' => __('Enter your merchant ID. You may need to contact TransFirst to obtain this information. <strong>This is not your Transaction Central login ID.</strong>', 'woocommerce_transfirst'),
						'default' => ''
					),
					'reg_key' => array (
						'title' => __('Reg Key (Password)', 'woocommerce_transfirst'),
						'type' => 'text',
						'description' => __('Enter the reg key associated with your merchant ID. <strong>This is not your Transaction Central login password.</strong>', 'woocommerce_transfirst'),
						'default' => ''
					),
					'invoice_prefix' => array (
						'title' => __('Invoice Prefix', 'woocommerce'),
						'type' => 'text',
						'description' => __('Please enter a prefix for your invoice numbers. If you use your TransFirst account for multiple stores, ensure this prefix is unique for each store.', 'woocommerce_transfirst'),
						'default' => 'WC-',
						'desc_tip' => true
					)
				);
			}

			/**
			 * Retrieves an array of arguments that get passed to the TransFirst API.
			 *
			 * @access public
			 * @param WC_Order $order The order object
			 * @return array The name/value argument pairs
			 */
			public function get_transfirst_args ($order) {
				return apply_filters('woocommerce_transfirst_args', array (
					'TransType' => 'CC',

					'MerchantID' => $this->merchant_id,
					'RegKey' => $this->reg_key,

					'NameonAccount' => $order->billing_first_name.' '.$order->billing_last_name,
					'AVSADDR' => ($order->billing_company ? $order->billing_company.', ' : '').$order->billing_address_1.($order->billing_address_2 ? ', '.$order->billing_address_2 : '').$order->billing_city.', '.$order->billing_state.' '.$order->billing_postcode.' '.$order->billing_country,
					'AVSZIP' => $order->billing_postcode,

					'AccountNo' => $_POST['AccountNo'],
					'CCMonth' => $_POST['CCMonth'],
					'CCYear' => $_POST['CCYear'],
					'CVV2' => $_POST['CVV2'],

					'RefID' => $this->invoice_prefix.ltrim($order->get_order_number(), '#'),
					'Amount' => number_format($order->get_total(), 2, '.', '')
				));
			}

			/**
			 * Outputs the credit card information fields.
			 *
			 * @access public
			 * @return void
			 */
			public function payment_fields () {
				?>
				<p class="form-row form-row-first validate-required">
					<label for="transfirst_AccountNo"><?php _e('Credit Card No.:', 'woocommerce_transfirst'); ?></label>
					<input type="text" class="input-text" name="AccountNo" id="transfirst_AccountNo">
				</p>
				<p class="form-row form-row-last validate-required">
					<label for="transfirst_CVV2"><?php _e('CVV2 Code:', 'woocommerce_transfirst'); ?></label>
					<input type="text" class="input-text" name="CVV2" id="transfirst_CVV2">
				</p>
				<div class="clear"></div>
				<p class="form-row form-row-first validate-required">
					<label for="transfirst_CCMonth"><?php _e('Expiration Month:', 'woocommerce_transfirst'); ?></label>
					<select name="CCMonth" id="transfirst_CCMonth">
						<option value=""></option>
				<?php
				for ($m = 1; $m <= 12; ++$m) {
				?>
						<option value="<?php print ($mm = str_pad($m, 2, '0', STR_PAD_LEFT)); ?>"><?php print $mm; ?></option>
				<?php
				}
				?>
					</select>
				</p>
				<p class="form-row form-row-last validate-required">
					<label for="transfirst_CCYear"><?php _e('Expiration Year:', 'woocommerce_transfirst'); ?></label>
					<select name="CCYear" id="transfirst_CCYear">
						<option value=""></option>
					<?php
					$thisYear = (int) date('y');
					for ($yy = $thisYear; $yy <= $thisYear+15; ++$yy) {
					?>
						<option value="<?php print $yy; ?>"><?php print ($yy+2000); ?></option>
					<?php
					}
					?>
					</select>
				</p>
				<div class="clear"></div>
				<?php
			}

			/**
			 * Sends the data to the TransFirst API for processing and handles the result.
			 *
			 * @access public
			 * @return array On success|void On error
			 */
			public function process_payment ($order_id) {
				global $woocommerce;
				$order = new WC_Order($order_id);

				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, 'https://webservices.primerchants.com/billing/TransactionCentral/processCC.asp?');
				curl_setopt($ch, CURLOPT_VERBOSE, 0);
				curl_setopt($ch, CURLOPT_POST, 1);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
				curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
				curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($this->get_transfirst_args($order), '', '&'));
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

				parse_str(curl_exec($ch), $response);
				if (!empty($response['Auth']) && $response['Auth'] != 'Declined') {
					$order->reduce_order_stock();
					$woocommerce->cart->empty_cart();
					$order->payment_complete();
					return array (
						'result' => 'success',
						'redirect' => $this->get_return_url($order)
					);
				}

				wc_add_notice(__('Payment error:', 'woothemes').' '.htmlentities($response['Notes']), 'error');
			}
		}
	}
	add_action('plugins_loaded', 'wc_transfirst_gateway_plugins_loaded');

	/**
	 * Adds the gateway after it is loaded.
	 *
	 * @access public
	 * @param array $methods Array of gateway classes
	 * @return array Array of gateway classes
	 */
	function wc_transfirst_gateway_woocommerce_payment_gateways ($gateways) {
		return array_merge($gateways, array ('WC_Gateway_TransFirst'));
	}
	add_action('woocommerce_payment_gateways', 'wc_transfirst_gateway_woocommerce_payment_gateways');
} else if (!function_exists('wc_transfirst_gateway_admin_notices')) {
	function wc_transfirst_gateway_admin_notices () {
		?>
		<div class="error fade"><?php _e('The WooCommerce TransFirst Gateway plugin could not be loaded because a class with the same name has already been defined.', 'woocommerce_transfirst'); ?></div>
		<?php
	}
	add_action('admin_notices', 'wc_transfirst_gateway_admin_notices');
}
?>